==> osso-abook-import-many_0.1-1.tar.gz: osso-abook-import-many-0.1/src/osso-abook-import-many.c <==
/* Import many contacts into Maemo 5 Contacts within one step.
 * Copyright (C) 2009  Mathias Hasselmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "config.h"

#include <dbus/dbus-glib.h>
#include <glib/gi18n.h>
#include <hildon/hildon.h>
#include <hildon/hildon-fm.h>

typedef struct {
  GFile         *folder;
  GList         *input_files;
  GList         *output_files;
  GCancellable  *cancellable;
  GError        *error;

  GOutputStream *stream;
  int            n_unique;
  int            n_contacts;

  char          *contents;
  gsize          offset;
  gsize          length;

  GtkWidget     *dialog;
  GtkWidget     *progress;
} MergeData;

static void
merge_next_file (MergeData *data);

static void
write_cb (GObject      *object,
          GAsyncResult *result,
          gpointer      user_data)
{
  MergeData     *data = user_data;
  GOutputStream *stream;
  gssize         written;

  stream = G_OUTPUT_STREAM (object);
  written = g_output_stream_write_finish (stream, result, &data->error);

  if (written >= 0)
    {
      data->offset += written;
      data->length -= written;

      if (data->length)
        {
          g_output_stream_write_async (stream, data->contents + data->offset,
                                       data->length, G_PRIORITY_DEFAULT,
                                       data->cancellable, write_cb, data);
        }
      else
        merge_next_file (data);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static GOutputStream *
create_unique_file (MergeData *data)
{
  GFileOutputStream *stream;
  GFile             *parent;
  char              *basename;

  basename = g_file_get_basename (data->folder);
  parent   = g_file_get_parent (data->folder);

  do
    {
      char   *unique;
      GFile  *file;

      if (!data->n_unique)
        unique = g_strdup_printf ("%s.vcf", basename);
      else
        unique = g_strdup_printf ("%s (%d).vcf", basename, data->n_unique);

      data->n_unique += 1;
      file = g_file_get_child (parent, unique);

      stream = g_file_create (file, G_FILE_CREATE_NONE,
                              data->cancellable, &data->error);

      if (stream)
        data->output_files = g_list_prepend (data->output_files, file);
      else if (data->error && g_error_matches (data->error, G_IO_ERROR, G_IO_ERROR_EXISTS))
        g_clear_error (&data->error);

      if (!stream)
        g_object_unref (file);

      g_free (unique);
    }
  while (!stream && !data->error);

  g_object_unref (parent);
  g_free (basename);

  return G_OUTPUT_STREAM (stream);
}

static void
load_contents_cb (GObject      *object,
                  GAsyncResult *result,
                  gpointer      user_data)
{
  MergeData *data = user_data;

  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (data->progress));

  g_free (data->contents);
  data->contents = NULL;
  data->offset = 0;
  data->length = 0;

  if (g_file_load_contents_finish (G_FILE (object), result,
                                   &data->contents, &data->length,
                                   NULL, &data->error))
    {
      if (data->stream && 0 == (data->n_contacts % 1000))
        {
          g_object_unref (data->stream);
          data->stream = NULL;
        }

      if (!data->stream)
        data->stream = create_unique_file (data);

      if (data->stream)
        {
          if ('\n' != data->contents[data->length - 1])
            {
              data->contents = g_realloc (data->contents, data->length + 1);
              data->contents[data->length++] = '\n';
            }

          g_output_stream_write_async (data->stream, data->contents, data->length,
                                       G_PRIORITY_DEFAULT, data->cancellable,
                                       write_cb, data);

          data->n_contacts += 1;
        }
    }

  if (!data->stream)
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static void
merge_next_file (MergeData *data)
{
  GFile *file = NULL;

  while (data->input_files && !file)
    {
      GFileInfo *info = data->input_files->data;

      if (g_strcmp0 ("text/vcard", g_file_info_get_content_type (info)))
        {
          file = g_file_get_child (data->folder, g_file_info_get_name (info));
          gtk_progress_bar_set_text (GTK_PROGRESS_BAR (data->progress),
                                     g_file_info_get_display_name (info));
        }

      data->input_files = g_list_delete_link (data->input_files, data->input_files);
      g_object_unref (info);
    }

  if (file)
    {
      g_file_load_contents_async (file, data->cancellable, load_contents_cb, data);
      g_object_unref (file);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK);
}

static void
next_files_cb (GObject      *object,
               GAsyncResult *result,
               gpointer      user_data)
{
  MergeData *data = user_data;
  GList     *files;

  files = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (object), result, &data->error);

  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (data->progress));

  if (files)
    {
      data->input_files = g_list_concat (files, data->input_files);

      g_file_enumerator_next_files_async (G_FILE_ENUMERATOR (object),
                                          G_MAXINT, G_PRIORITY_DEFAULT,
                                          data->cancellable, next_files_cb, data);
    }
  else if (data->error)
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
  else
    merge_next_file (data);
}

static void
enumerate_children_cb (GObject      *object,
                       GAsyncResult *result,
                       gpointer      user_data)
{
  MergeData       *data = user_data;
  GFileEnumerator *enumerator;

  enumerator = g_file_enumerate_children_finish (G_FILE (object), result, &data->error);

  if (enumerator)
    {
      g_file_enumerator_next_files_async (enumerator, G_MAXINT, G_PRIORITY_DEFAULT,
                                          data->cancellable, next_files_cb, data);
      g_object_unref (enumerator);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static gboolean
confirm_import (GtkWindow       *parent,
                const MergeData *data)
{
  GtkWidget *dialog;
  int        response;
  int        n_output_files;
  char      *text;

  n_output_files = g_list_length (data->output_files);

  text = g_strdup_printf ("Finished with merging %d %s into %d vCard %s.\n"
                          "Do you wish to import this files into Contacts now?",
                          data->n_contacts, ngettext("contact", "contacts", data->n_contacts),
                          n_output_files, ngettext("file", "files", n_output_files));

  dialog = hildon_note_new_confirmation (parent, text);
  response = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
  g_free (text);

  return (GTK_RESPONSE_OK == response);
}

static void
object_list_free (GList *list)
{
  g_list_foreach (list, (GFunc) g_object_unref, NULL);
  g_list_free (list);
}

static GList *
merge_contact_files (GtkWindow *parent,
                     GFile     *folder)
{
  MergeData data = { 0, };

  data.folder = folder;
  data.cancellable = g_cancellable_new ();
  data.progress = gtk_progress_bar_new ();

  data.dialog = hildon_note_new_cancel_with_progress_bar (parent, "Merging contacts files...",
                                                          GTK_PROGRESS_BAR (data.progress));

  g_file_enumerate_children_async (folder, G_FILE_ATTRIBUTE_STANDARD_NAME ","
                                   G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
                                   G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                   G_PRIORITY_DEFAULT, data.cancellable,
                                   enumerate_children_cb, &data);

  if (GTK_RESPONSE_OK != gtk_dialog_run (GTK_DIALOG (data.dialog)))
    {
      g_cancellable_cancel (data.cancellable);
      object_list_free (data.output_files);
      data.output_files = NULL;
    }

  if (data.error)
    {
      hildon_banner_show_information (NULL, NULL, data.error->message);
      g_error_free (data.error);
    }

  object_list_free (data.input_files);

  if (data.stream)
    g_object_unref (data.stream);

  g_object_unref (data.cancellable);
  gtk_widget_destroy (data.dialog);
  g_free (data.contents);

  if (!confirm_import (parent, &data))
    {
      object_list_free (data.output_files);
      data.output_files = NULL;
    }

  return data.output_files;
}

static char *
get_mydocs_path (void)
{
  char *mydocs = g_strdup (g_getenv ("MYDOCSDIR"));

  if (!mydocs)
    {
      const char *home = g_getenv ("HOME");

      if (!home)
        home = g_get_home_dir ();

      mydocs = g_build_filename (home, "MyDocs", NULL);
    }

  return mydocs;
}

static GFile *
select_contacts_folder (GtkWindow *parent)
{
  GFile     *folder = NULL;
  char      *mydocs;
  GtkWidget *dialog;

  dialog = hildon_file_chooser_dialog_new (parent, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
  g_object_set (dialog, "title", "Select Contacts Folder", "show-folder-button", FALSE, NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), mydocs = get_mydocs_path ());

  if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog)))
    folder = gtk_file_chooser_get_current_folder_file (GTK_FILE_CHOOSER (dialog));

  gtk_widget_destroy (dialog);
  g_free (mydocs);

  return folder;
}

static void
import_merge_files (GtkWindow *window,
                    GList     *file_list)
{
  DBusGConnection *session;
  DBusGProxy      *proxy;
  GError          *error = NULL;
  char            *path = NULL;

  session = dbus_g_bus_get (DBUS_BUS_SESSION, &error);

  if (!session)
    goto out;

  proxy = dbus_g_proxy_new_for_name_owner (session,
                                           "com.nokia.osso_addressbook",
                                           "/com/nokia/osso_addressbook",
                                           "com.nokia.osso_addressbook",
                                           &error);

  if (!proxy)
    goto out;

  while (file_list)
    {
      char *uri = g_file_get_uri (file_list->data);
      gboolean success;

      success = dbus_g_proxy_call (proxy, "mime_open", &error,
                                   G_TYPE_STRING, uri, G_TYPE_INVALID,
                                   G_TYPE_INVALID);

      g_free (uri);

      if (!success)
        goto out;

      file_list = file_list->next;
    }

out:
  if (error)
    {
      hildon_banner_show_information (NULL, NULL, error->message);
      g_error_free (error);
    }

  if (proxy)
    g_object_unref (proxy);
}

int
main (int    argc,
      char **argv)
{
  GFile *folder = NULL;
  GList *merge_files;

  hildon_gtk_init (&argc, &argv);

  folder = select_contacts_folder (NULL);

  if (folder)
    merge_files = merge_contact_files (NULL, folder);
  if (merge_files)
    import_merge_files (NULL, merge_files);

  object_list_free (merge_files);

  if (folder)
    g_object_unref (folder);

  return 0;
}

==> osso-abook-import-many_0.2-1.tar.gz: osso-abook-import-many-0.2/src/osso-abook-import-many.c <==
/* Import many contacts into Maemo 5 Contacts within one step.
 * Copyright (C) 2009  Mathias Hasselmann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "config.h"

#include <dbus/dbus-glib.h>
#include <glib/gi18n.h>
#include <hildon/hildon.h>
#include <hildon/hildon-fm.h>

typedef struct {
  GFile         *folder;
  GList         *input_files;
  GList         *output_files;
  GCancellable  *cancellable;
  GError        *error;

  GOutputStream *stream;
  int            n_unique;
  int            n_contacts;

  char          *contents;
  gsize          offset;
  gsize          length;

  GtkWidget     *dialog;
  GtkWidget     *progress;
} MergeData;

static void
merge_next_file (MergeData *data);

static void
write_cb (GObject      *object,
          GAsyncResult *result,
          gpointer      user_data)
{
  MergeData     *data = user_data;
  GOutputStream *stream;
  gssize         written;

  stream = G_OUTPUT_STREAM (object);
  written = g_output_stream_write_finish (stream, result, &data->error);

  if (written >= 0)
    {
      data->offset += written;
      data->length -= written;

      if (data->length)
        {
          g_output_stream_write_async (stream, data->contents + data->offset,
                                       data->length, G_PRIORITY_DEFAULT,
                                       data->cancellable, write_cb, data);
        }
      else
        merge_next_file (data);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static GOutputStream *
create_unique_file (MergeData *data)
{
  GFileOutputStream *stream;
  GFile             *parent;
  char              *basename;

  basename = g_file_get_basename (data->folder);
  parent   = g_file_get_parent (data->folder);

  do
    {
      char   *unique;
      GFile  *file;

      if (!data->n_unique)
        unique = g_strdup_printf ("%s.vcf", basename);
      else
        unique = g_strdup_printf ("%s (%d).vcf", basename, data->n_unique);

      data->n_unique += 1;
      file = g_file_get_child (parent, unique);

      stream = g_file_create (file, G_FILE_CREATE_NONE,
                              data->cancellable, &data->error);

      if (stream)
        data->output_files = g_list_prepend (data->output_files, file);
      else if (data->error && g_error_matches (data->error, G_IO_ERROR, G_IO_ERROR_EXISTS))
        g_clear_error (&data->error);

      if (!stream)
        g_object_unref (file);

      g_free (unique);
    }
  while (!stream && !data->error);

  g_object_unref (parent);
  g_free (basename);

  return G_OUTPUT_STREAM (stream);
}

static void
load_contents_cb (GObject      *object,
                  GAsyncResult *result,
                  gpointer      user_data)
{
  MergeData *data = user_data;

  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (data->progress));

  g_free (data->contents);
  data->contents = NULL;
  data->offset = 0;
  data->length = 0;

  if (g_file_load_contents_finish (G_FILE (object), result,
                                   &data->contents, &data->length,
                                   NULL, &data->error))
    {
      if (data->stream && 0 == (data->n_contacts % 1000))
        {
          g_object_unref (data->stream);
          data->stream = NULL;
        }

      if (!data->stream)
        data->stream = create_unique_file (data);

      if (data->stream)
        {
          if ('\n' != data->contents[data->length - 1])
            {
              data->contents = g_realloc (data->contents, data->length + 1);
              data->contents[data->length++] = '\n';
            }

          g_output_stream_write_async (data->stream, data->contents, data->length,
                                       G_PRIORITY_DEFAULT, data->cancellable,
                                       write_cb, data);

          data->n_contacts += 1;
        }
    }

  if (!data->stream)
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static void
merge_next_file (MergeData *data)
{
  GFile *file = NULL;

  while (data->input_files && !file)
    {
      GFileInfo *info = data->input_files->data;

      if (g_strcmp0 ("text/vcard", g_file_info_get_content_type (info)))
        {
          file = g_file_get_child (data->folder, g_file_info_get_name (info));
          gtk_progress_bar_set_text (GTK_PROGRESS_BAR (data->progress),
                                     g_file_info_get_display_name (info));
        }

      data->input_files = g_list_delete_link (data->input_files, data->input_files);
      g_object_unref (info);
    }

  if (file)
    {
      g_file_load_contents_async (file, data->cancellable, load_contents_cb, data);
      g_object_unref (file);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK);
}

static void
next_files_cb (GObject      *object,
               GAsyncResult *result,
               gpointer      user_data)
{
  MergeData *data = user_data;
  GList     *files;

  files = g_file_enumerator_next_files_finish (G_FILE_ENUMERATOR (object), result, &data->error);

  gtk_progress_bar_pulse (GTK_PROGRESS_BAR (data->progress));

  if (files)
    {
      data->input_files = g_list_concat (files, data->input_files);

      g_file_enumerator_next_files_async (G_FILE_ENUMERATOR (object),
                                          G_MAXINT, G_PRIORITY_DEFAULT,
                                          data->cancellable, next_files_cb, data);
    }
  else if (data->error)
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
  else
    merge_next_file (data);
}

static void
enumerate_children_cb (GObject      *object,
                       GAsyncResult *result,
                       gpointer      user_data)
{
  MergeData       *data = user_data;
  GFileEnumerator *enumerator;

  enumerator = g_file_enumerate_children_finish (G_FILE (object), result, &data->error);

  if (enumerator)
    {
      g_file_enumerator_next_files_async (enumerator, G_MAXINT, G_PRIORITY_DEFAULT,
                                          data->cancellable, next_files_cb, data);
      g_object_unref (enumerator);
    }
  else
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

static void
report_success (GtkWindow       *parent,
                const MergeData *data)
{
  int n_output_files = g_list_length (data->output_files);

  hildon_banner_show_informationf ((GtkWidget *) parent, NULL,
                                   "Successfully merged %d %s into %d vCard %s.",
                                   data->n_contacts, ngettext("contact", "contacts", data->n_contacts),
                                   n_output_files, ngettext("file", "files", n_output_files));
}

static void
object_list_free (GList *list)
{
  g_list_foreach (list, (GFunc) g_object_unref, NULL);
  g_list_free (list);
}

static gboolean
merge_contact_files (GtkWindow *parent,
                     MergeData *data)
{
  gboolean success = TRUE;

  data->progress = gtk_progress_bar_new ();

  data->dialog = hildon_note_new_cancel_with_progress_bar (parent, "Merging contacts files...",
                                                           GTK_PROGRESS_BAR (data->progress));

  g_file_enumerate_children_async (data->folder, G_FILE_ATTRIBUTE_STANDARD_NAME ","
                                   G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
                                   G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                                   G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
                                   G_PRIORITY_DEFAULT, data->cancellable,
                                   enumerate_children_cb, data);

  if (GTK_RESPONSE_OK != gtk_dialog_run (GTK_DIALOG (data->dialog)))
    {
      g_cancellable_cancel (data->cancellable);
      object_list_free (data->output_files);
      data->output_files = NULL;
      success = FALSE;
    }

  if (data->error)
    {
      hildon_banner_show_information (NULL, NULL, data->error->message);
      g_clear_error (&data->error);
    }

  object_list_free (data->input_files);

  if (data->stream)
    data->stream = (g_object_unref (data->stream), NULL);

  gtk_widget_destroy (data->dialog);
  data->progress = NULL;
  data->dialog = NULL;

  g_free (data->contents);
  data->contents = NULL;

  if (success)
    report_success (parent, data);

  return success;
}

static char *
get_mydocs_path (void)
{
  char *mydocs = g_strdup (g_getenv ("MYDOCSDIR"));

  if (!mydocs)
    {
      const char *home = g_getenv ("HOME");

      if (!home)
        home = g_get_home_dir ();

      mydocs = g_build_filename (home, "MyDocs", NULL);
    }

  return mydocs;
}

static GFile *
select_contacts_folder (GtkWindow *parent)
{
  GFile     *folder = NULL;
  char      *mydocs;
  GtkWidget *dialog;

  dialog = hildon_file_chooser_dialog_new (parent, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
  g_object_set (dialog, "title", "Select Contacts Folder", "show-folder-button", FALSE, NULL);
  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), mydocs = get_mydocs_path ());

  if (GTK_RESPONSE_OK == gtk_dialog_run (GTK_DIALOG (dialog)))
    folder = gtk_file_chooser_get_current_folder_file (GTK_FILE_CHOOSER (dialog));

  gtk_widget_destroy (dialog);
  g_free (mydocs);

  return folder;
}

static void
import_merge_files (GtkWindow *window,
                    GList     *file_list)
{
  DBusGConnection *session = NULL;
  DBusGProxy      *proxy   = NULL;
  GError          *error   = NULL;

  session = dbus_g_bus_get (DBUS_BUS_SESSION, &error);

  if (!session)
    goto out;

  proxy = dbus_g_proxy_new_for_name (session,
                                     "com.nokia.osso_addressbook",
                                     "/com/nokia/osso_addressbook",
                                     "com.nokia.osso_addressbook");

  while (file_list)
    {
      char *uri = g_file_get_uri (file_list->data);
      gboolean success;

      success = dbus_g_proxy_call (proxy, "mime_open", &error,
                                   G_TYPE_STRING, uri, G_TYPE_INVALID,
                                   G_TYPE_INVALID);

      g_free (uri);

      if (!success)
        goto out;

      file_list = file_list->next;
    }

out:
  if (error)
    {
      hildon_banner_show_information (NULL, NULL, error->message);
      g_error_free (error);
    }

  if (proxy)
    g_object_unref (proxy);
}

static void
map_cb (GtkWidget *window)
{
  hildon_gtk_window_take_screenshot (GTK_WINDOW (window), TRUE);
}

static void
destroy_cb (MergeData *data)
{
  if (data->dialog)
    gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);

  gtk_main_quit ();
}

static void
merge_contacts_cb (GtkWidget *button,
                   MergeData *data)
{
  GtkWidget *window;

  window = gtk_widget_get_ancestor (button, GTK_TYPE_WINDOW);
  data->folder = select_contacts_folder (GTK_WINDOW (window));

  if (data->folder)
    merge_contact_files (GTK_WINDOW (window), data);

  object_list_free (data->output_files);
  data->output_files = NULL;

  if (data->folder)
    data->folder = (g_object_unref (data->folder), NULL);
}

static void
import_contacts_cb (GtkWidget *button,
                    MergeData *data)
{
  GtkWidget *window;

  window = gtk_widget_get_ancestor (button, GTK_TYPE_WINDOW);
  data->folder = select_contacts_folder (GTK_WINDOW (window));

  if (data->folder && merge_contact_files (GTK_WINDOW (window), data))
    import_merge_files (GTK_WINDOW (window), data->output_files);

  object_list_free (data->output_files);
  data->output_files = NULL;

  if (data->folder)
    data->folder = (g_object_unref (data->folder), NULL);
}

static GtkWidget *
create_button (const char *icon_name,
               const char *title)
{
  GtkWidget *button, *vbox, *image, *label;

  image = gtk_image_new_from_icon_name (icon_name, HILDON_ICON_SIZE_XLARGE);

  label = gtk_label_new (title);
  hildon_helper_set_logical_font (label, "LargeSystemFont");

  vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_DEFAULT);
  gtk_container_add (GTK_CONTAINER (vbox), image);
  gtk_container_add (GTK_CONTAINER (vbox), label);
  gtk_widget_show_all (vbox);

  button = gtk_button_new ();
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  gtk_container_add (GTK_CONTAINER (button), vbox);

  return button;
}

static GtkWidget *
create_main_window (MergeData *data)
{
  GtkWidget *window, *align, *hbox, *button;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  g_signal_connect_swapped (window, "destroy", G_CALLBACK (destroy_cb), data);
  g_signal_connect (window, "map", G_CALLBACK (map_cb), data);

  align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
  gtk_container_add (GTK_CONTAINER (window), align);

  hbox = gtk_hbox_new (TRUE, HILDON_MARGIN_DOUBLE);
  gtk_container_add (GTK_CONTAINER (align), hbox);

  button = create_button ("osso-abook-merge-many", "Merge Contact Files");
  g_signal_connect (button, "clicked", G_CALLBACK (merge_contacts_cb), data);
  gtk_container_add (GTK_CONTAINER (hbox), button);

  button = create_button ("osso-abook-import-many", "Import Contact Files");
  g_signal_connect (button, "clicked", G_CALLBACK (import_contacts_cb), data);
  gtk_container_add (GTK_CONTAINER (hbox), button);

  gtk_widget_show_all (window);

  return window;
}

int
main (int    argc,
      char **argv)
{
  MergeData  data = { 0, };
  GtkWidget *window;

  g_set_application_name ("Import Contacts");
  hildon_gtk_init (&argc, &argv);

  data.cancellable = g_cancellable_new ();

  window = create_main_window (&data);
  g_object_add_weak_pointer (G_OBJECT (window), (gpointer) &window);

  gtk_main ();

  if (window)
    gtk_widget_destroy (window);

  g_object_unref (data.cancellable);

  return 0;
}
